import 'package:characters/characters.dart';

import './key_space.dart';

/// Get the string between two strings in a lexicographical order.
String between({String prev = '', String next = ''}) {
  String assertionInfo(String message) {
    return "$message\n"
        "prev: $prev\n"
        "next: $next\n"
        "avalables keys: ${keys.join('')}";
  }

  assert(
    () {
      bool _isConfiguredWithOnlyAvailableKeys(String value) {
        return Characters(value).every((x) => keys.contains(x));
      }

      return _isConfiguredWithOnlyAvailableKeys(prev) &&
          _isConfiguredWithOnlyAvailableKeys(next);
    }(),
    assertionInfo("'prev' and 'next' should consist only of allowed keys.\n"),
  );
  assert(next != keys.first, assertionInfo("'next' shouldn't be 'A'"));
  assert(
    prev.isNotEmpty && next.isNotEmpty,
    assertionInfo("'prev' and 'next' shouldn't be empty"),
  );
  assert(
    prev != next,
    assertionInfo("'prev' and 'next' shouldn't be equal to each other"),
  );
  assert(
    prev.compareTo(next) == -1,
    assertionInfo(
        "'prev' should not precede 'next' in a lexicographical order"),
  );

  late int _prev, _next;
  int pos = 0;
  String str;
  for (; pos == 0 || _prev == _next; pos++) {
    _prev = pos < prev.length ? keyToIndex[prev[pos]]! : -1;
    _next = pos < next.length ? keyToIndex[next[pos]]! : keyToIndex.length;
  }
  str = prev.substring(0, pos - 1);
  if (_prev == -1) {
    while (_next == 0) {
      _next = pos < next.length ? keyToIndex[next[pos++]]! : keyToIndex.length;
      str += keys.first;
    }
    if (_next == 1) {
      str += keys.first;
      _next = keyToIndex.length;
    }
  } else if (_prev + 1 == _next) {
    str += keys[_prev];
    _next = keyToIndex.length;
    while ((_prev = pos < prev.length ? keyToIndex[prev[pos++]]! : -1) ==
        keyToIndex.length - 1) {
      str += keys.last;
    }
  }
  return str + keys[((_prev + _next) / 2).ceil()];
}
